require_relative "player"
require_relative "game_turn"
require_relative "treasure_trove"

module StudioGame
  class Game
    attr_reader :title

    def initialize(title)
      @title = title.capitalize
      @players = []
    end

    def add_player(player)
      @players.push(player)
    end

    def load_players(from_file)
      File.readlines(from_file).each do |line|
        add_player(Player.from_csv(line)) 
      end
    end

    def high_score_entry(player)
      "#{player.name.ljust(20,'.')}#{player.score}"
    end

    def save_high_scores(to_file = "high_scores.txt")
      File.open(to_file, "w") do |file| 
        file.puts "#{@title} High Scores:" 
        @players.sort.each do |player|
          file.puts high_score_entry(player)
        end 
      end
    end

    def play(rounds)
      puts "There are #{@players.size} players in #{@title}"
      puts @players
      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures available."
      treasures.each do |treasure|
        puts "#{treasure.name.to_s.capitalize.ljust(30, ".")}#{treasure.points} points"
      end
      1.upto(rounds) do |count|
        puts "\nRound Number #{count}:"
        @players.each do |player|
          GameTurn.take_turn(player)
        end 
      end   
    end

    def print_stats
      strong_players, wimpy_players = @players.partition { |player| player.strong? }
      puts "\n#{@title} Statistics:"
      puts "\n#{strong_players.size} Strong Players:"
      print_strong_and_wimpy_stats(strong_players)
      puts "\n#{wimpy_players.size} Wimpy Players:"
      print_strong_and_wimpy_stats(wimpy_players)
      puts "\n#{@title} High Scores:"
      @players.sort.each do |player|
        puts high_score_entry(player)
      end  
      puts "\nPlayer's Points:"
      @players.sort.each do |player|
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.name.ljust(20,'.')}#{player.points} points"
      end 
      puts "\nTotal Points:"
      puts "#{total_points} total points from treasures found"
    end

    def print_strong_and_wimpy_stats(players)
      players.each do |player|
        puts "#{player.name}: #{player.health}"
      end
    end

    def total_points
      @players.reduce(0) { |sum, player| sum + player.points}
    end

  end
end